@@ -1,30 +1,18 @@
-from .commands import list_all, stop, play, status, id_from_index_or_name, \
-        next, previous, pause, count, id, name, ip, signal_strength, \
-        connected, synced_to, sync_to, sync_groups, get_volume, set_volume, \
+from .commands import stop, play, status, next, previous, pause, \
+        sync_to, sync_groups, get_volume, set_volume, \
         change_volume, mute, unmute, toggle_mute, get_mute, players, \
-        listen, playlist_add, playlist_remove, playlist_insert, Shuffle, \
+        playlist_add, playlist_remove, playlist_insert, Shuffle, \
         playlist_query_shuffle, playlist_set_shuffle, Repeat, \
         playlist_query_repeat, playlist_set_repeat, playlist_toggle_repeat, \
-        playlist_toggle_shuffle
-from .notifications import pause_handler, volume_handler, newsong_handler, \
-        playlistcontrol_handler, play_handler
+        playlist_toggle_shuffle, playlist_index
 
 __all__ = (
-    'list_all',
     'stop',
     'play',
     'status',
-    'id_from_index_or_name',
     'next',
     'previous',
     'pause',
-    'count',
-    'id',
-    'name',
-    'ip',
-    'signal_strength',
-    'connected',
-    'synced_to',
     'sync_to',
     'sync_groups',
     'get_volume',
@@ -47,8 +35,5 @@ __all__ = (
     'playlist_query_repeat',
     'playlist_set_repeat',
     'playlist_toggle_repeat',
+    'playlist_index',
     )
-
-notification_handlers = [
-        pause_handler, volume_handler, newsong_handler,
-        playlistcontrol_handler, play_handler]
