@@ -1,16 +1,9 @@
 import pytest
-import socket
 from click.testing import CliRunner
 
 import squeezebox_cli.ui.main
 
 
-@pytest.fixture()
-def telnet(mocker):
-    return mocker.patch('squeezebox_cli.ui.main.Telnet',
-                        return_value=mocker.MagicMock())
-
-
 @pytest.fixture()
 def min_config(mocker):
     return mocker.patch('squeezebox_cli.ui.main.open',
@@ -18,16 +11,10 @@ def min_config(mocker):
                                                    b"host = 'my-host'\n"))
 
 
-@pytest.fixture()
-def player_from_index_or_name(mocker):
-    return mocker.patch('squeezebox_cli.ui.main.id_from_index_or_name',
-                        return_value='12:34:56:78:90')
-
-
 @pytest.fixture()
 def player_status(mocker):
     return mocker.patch(
-            'squeezebox_cli.ui.main.status',
+            'squeezebox_cli.ui.main.squeezebox_cli.player.status',
             return_value={
                  'name': 'office',
                  'mode': 'stop',
@@ -72,55 +59,58 @@ def database_songinfo(mocker):
                 'artist': 'Taylor Swift',
                 },
             }
-    return mocker.patch('squeezebox_cli.ui.main.songinfo',
-                        lambda sb_server, track_id: stub_songinfo[track_id])
+    return mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.database.songinfo',
+            lambda host_port, track_id: stub_songinfo[track_id])
 
 
 @pytest.fixture()
 def playlist_add(mocker):
-    return mocker.patch('squeezebox_cli.ui.main.playlist_add')
+    return mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_add')
 
 
 @pytest.fixture()
 def playlist_insert(mocker):
-    return mocker.patch('squeezebox_cli.ui.main.playlist_insert')
+    return mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_insert')
 
 
 @pytest.fixture()
 def player_set_volume(mocker):
-    return mocker.patch('squeezebox_cli.ui.main.set_volume')
+    return mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.set_volume')
 
 
 @pytest.fixture()
 def players_list_all(mocker):
     return mocker.patch(
-            'squeezebox_cli.ui.main.list_all',
+            'squeezebox_cli.ui.main.squeezebox_cli.player.players',
             return_value=[
                 dict(playerindex=0,
-                     playerid='00:0f:00:64:6d:da',
+                     connected=True,
                      name='office',
                      isplaying=False),
                 dict(playerindex=1,
-                     playerid='00:04:20:23:30:7f',
+                     connected=True,
                      name='lounge',
                      isplaying=True),
                 ])
 
 
-def test_players_valid_connection_args(mocker, telnet, players_list_all):
+def test_players_valid_connection_args(mocker, players_list_all):
     runner = CliRunner()
     result = runner.invoke(squeezebox_cli.ui.main.ui_main,
                            ['--host=my-host', '--port=1234', 'players'])
-    telnet.assert_called_once_with('my-host', 1234)
-    players_list_all.assert_called_once_with(telnet.return_value)
+    players_list_all.assert_called_once_with(('my-host', 1234))
     assert 0 == result.exit_code
-    assert ('  index  name    is playing?\n'
-            '-------  ------  -------------\n'
-            '      0  office  no\n'
-            '      1  lounge  yes\n') == result.output
+    assert ('name    is playing?\n'
+            '------  -------------\n'
+            'office  no\n'
+            'lounge  yes\n') == result.output
 
 
-def test_players_valid_connection_config(mocker, telnet, players_list_all):
+def test_players_valid_connection_config(mocker, players_list_all):
     open_config = mocker.patch('squeezebox_cli.ui.main.open',
                                mocker.mock_open(read_data=b'[server]\n'
                                                           b"host = 'my-host'\n"
@@ -128,17 +118,16 @@ def test_players_valid_connection_config(mocker, telnet, players_list_all):
     runner = CliRunner()
     result = runner.invoke(squeezebox_cli.ui.main.ui_main, ['players'])
     open_config.assert_called_once()
-    telnet.assert_called_once_with('my-host', 1234)
-    players_list_all.assert_called_once_with(telnet.return_value)
+    players_list_all.assert_called_once_with(('my-host', 1234))
     assert 0 == result.exit_code
-    assert ('  index  name    is playing?\n'
-            '-------  ------  -------------\n'
-            '      0  office  no\n'
-            '      1  lounge  yes\n') == result.output
+    assert ('name    is playing?\n'
+            '------  -------------\n'
+            'office  no\n'
+            'lounge  yes\n') == result.output
 
 
 def test_players_valid_connection_config_default_port(
-        mocker, telnet, players_list_all):
+        mocker, players_list_all):
     open_config = mocker.patch('squeezebox_cli.ui.main.open',
                                mocker.mock_open(
                                    read_data=b'[server]\n'
@@ -146,13 +135,12 @@ def test_players_valid_connection_config_default_port(
     runner = CliRunner()
     result = runner.invoke(squeezebox_cli.ui.main.ui_main, ['players'])
     open_config.assert_called_once()
-    telnet.assert_called_once_with('my-host', 9090)
-    players_list_all.assert_called_once_with(telnet.return_value)
+    players_list_all.assert_called_once_with(('my-host', 9000))
     assert 0 == result.exit_code
-    assert ('  index  name    is playing?\n'
-            '-------  ------  -------------\n'
-            '      0  office  no\n'
-            '      1  lounge  yes\n') == result.output
+    assert ('name    is playing?\n'
+            '------  -------------\n'
+            'office  no\n'
+            'lounge  yes\n') == result.output
 
 
 def test_players_invalid_connection_no_config(mocker):
@@ -168,31 +156,9 @@ def test_players_invalid_connection_no_config(mocker):
             ' or by argument (--host=).\n') == result.output
 
 
-def test_players_invalid_host(mocker):
-    tn = mocker.patch('squeezebox_cli.ui.main.Telnet',
-                      side_effect=socket.gaierror)
-    runner = CliRunner()
-    result = runner.invoke(squeezebox_cli.ui.main.ui_main,
-                           ['--host=my-host', '--port=1234', 'players'])
-    tn.assert_called_once_with('my-host', 1234)
-    assert 1 == result.exit_code
-    assert 'ERROR: could not connect to my-host:1234\n' == result.output
-
-
-def test_players_invalid_port(mocker):
-    tn = mocker.patch('squeezebox_cli.ui.main.Telnet',
-                      side_effect=ConnectionRefusedError())
-    runner = CliRunner()
-    result = runner.invoke(squeezebox_cli.ui.main.ui_main,
-                           ['--host=my-host', '--port=1234', 'players'])
-    tn.assert_called_once_with('my-host', 1234)
-    assert 1 == result.exit_code
-    assert 'ERROR: could not connect to my-host:1234\n' == result.output
-
-
-def test_search_nothing_found(mocker, min_config, telnet):
+def test_search_nothing_found(mocker, min_config):
     search = mocker.patch(
-            'squeezebox_cli.ui.main.search',
+            'squeezebox_cli.ui.main.squeezebox_cli.database.search',
             return_value={
                 'artists': {},
                 'albums': {},
@@ -200,7 +166,7 @@ def test_search_nothing_found(mocker, min_config, telnet):
                 })
     runner = CliRunner()
     result = runner.invoke(squeezebox_cli.ui.main.ui_main, ['search', 'toast'])
-    search.assert_called_once_with(telnet.return_value, 'toast')
+    search.assert_called_once_with(('my-host', 9000), 'toast')
     assert 0 == result.exit_code
     assert ('artist    id\n'
             '--------  ----\n'
@@ -212,26 +178,27 @@ def test_search_nothing_found(mocker, min_config, telnet):
             '-------  ----\n') == result.output
 
 
-def test_search_something_found(mocker, min_config, telnet):
-    search = mocker.patch('squeezebox_cli.ui.main.search',
-                          return_value={
-                              'artists': {
-                                  1234: 'A Singer',
-                                  1235: 'A Band',
-                                  1236: 'A Group',
-                                  },
-                              'albums': {
-                                  2345: 'A Rock Album',
-                                  2346: 'A Jazz Album',
-                                  },
-                              'tracks': {
-                                  3456: 'A Song',
-                                  3457: 'Another Song',
-                                  },
-                              })
+def test_search_something_found(mocker, min_config):
+    search = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.database.search',
+            return_value={
+                'artists': {
+                    1234: 'A Singer',
+                    1235: 'A Band',
+                    1236: 'A Group',
+                    },
+                'albums': {
+                    2345: 'A Rock Album',
+                    2346: 'A Jazz Album',
+                    },
+                'tracks': {
+                    3456: 'A Song',
+                    3457: 'Another Song',
+                    },
+                })
     runner = CliRunner()
     result = runner.invoke(squeezebox_cli.ui.main.ui_main, ['search', 'toast'])
-    search.assert_called_once_with(telnet.return_value, 'toast')
+    search.assert_called_once_with(('my-host', 9000), 'toast')
     assert 0 == result.exit_code
     assert ('artist      id\n'
             '--------  ----\n'
@@ -251,86 +218,77 @@ def test_search_something_found(mocker, min_config, telnet):
             ) == result.output
 
 
-def test_player_no_such_name(mocker, min_config, telnet):
-    from_index_or_name = mocker.patch(
-        'squeezebox_cli.ui.main.id_from_index_or_name', return_value=None)
+def test_player_no_such_name(mocker, min_config, players_list_all):
+    result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
+        'player',
+        'garden',
+        'stop',
+        ])
+    assert 'ERROR: no such player: garden\n' == result.output
+    assert 2 == result.exit_code
+
+
+def test_player_stop(mocker, min_config, players_list_all):
+    player_stop = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.stop')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'stop',
         ])
-    from_index_or_name.assert_called_once_with(telnet.return_value, 'office')
-    assert 1 == result.exit_code
-    assert 'ERROR: no such player: office\n' == result.output
-
-
-def test_player_stop(mocker, min_config, telnet, player_from_index_or_name):
-    player_stop = mocker.patch('squeezebox_cli.ui.main.stop')
-    result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
-        'player',
-        'office',
-        'stop',
-        ])
-    player_stop.assert_called_once_with(telnet.return_value,
-                                        player_from_index_or_name.return_value)
+    player_stop.assert_called_once_with(('my-host', 9000), 'office')
     assert 0 == result.exit_code
 
 
-def test_player_shuffled_none(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_shuffled_none(mocker, min_config, players_list_all):
     player_shuffled = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_query_shuffle',
-            return_value=squeezebox_cli.ui.main.Shuffle.NONE)
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_query_shuffle',
+            return_value=squeezebox_cli.player.Shuffle.NONE)
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'shuffled',
         ])
-    player_shuffled.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+    player_shuffled.assert_called_once_with(('my-host', 9000), 'office')
     assert 0 == result.exit_code
     assert 'none\n' == result.output
 
 
-def test_player_shuffled_song(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_shuffled_song(mocker, min_config, players_list_all):
     player_shuffled = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_query_shuffle',
-            return_value=squeezebox_cli.ui.main.Shuffle.SONG)
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_query_shuffle',
+            return_value=squeezebox_cli.player.Shuffle.SONG)
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'shuffled',
         ])
-    player_shuffled.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+    player_shuffled.assert_called_once_with(('my-host', 9000), 'office')
     assert 0 == result.exit_code
     assert 'song\n' == result.output
 
 
-def test_player_shuffled_album(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_shuffled_album(mocker, min_config, players_list_all):
     player_shuffled = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_query_shuffle',
-            return_value=squeezebox_cli.ui.main.Shuffle.ALBUM)
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_query_shuffle',
+            return_value=squeezebox_cli.player.Shuffle.ALBUM)
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'shuffled',
         ])
-    player_shuffled.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+    player_shuffled.assert_called_once_with(('my-host', 9000), 'office')
     assert 0 == result.exit_code
     assert 'album\n' == result.output
 
 
-def test_player_shuffle_none(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_shuffle_none(mocker, min_config, players_list_all):
     player_shuffle = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_set_shuffle')
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_set_shuffle')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -338,16 +296,16 @@ def test_player_shuffle_none(
         'none',
         ])
     player_shuffle.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            squeezebox_cli.ui.main.Shuffle.NONE)
+            ('my-host', 9000),
+            'office',
+            squeezebox_cli.player.Shuffle.NONE)
     assert 0 == result.exit_code
 
 
-def test_player_shuffle_song(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_shuffle_song(mocker, min_config, players_list_all):
     player_shuffle = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_set_shuffle')
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_set_shuffle')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -355,16 +313,16 @@ def test_player_shuffle_song(
         'song',
         ])
     player_shuffle.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            squeezebox_cli.ui.main.Shuffle.SONG)
+            ('my-host', 9000),
+            'office',
+            squeezebox_cli.player.Shuffle.SONG)
     assert 0 == result.exit_code
 
 
-def test_player_shuffle_album(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_shuffle_album(mocker, min_config, players_list_all):
     player_shuffle = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_set_shuffle')
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_set_shuffle')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -372,81 +330,81 @@ def test_player_shuffle_album(
         'album',
         ])
     player_shuffle.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            squeezebox_cli.ui.main.Shuffle.ALBUM)
+            ('my-host', 9000),
+            'office',
+            squeezebox_cli.player.Shuffle.ALBUM)
     assert 0 == result.exit_code
 
 
-def test_player_toggle_shuffle(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_toggle_shuffle(mocker, min_config, players_list_all):
     player_shuffle = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_toggle_shuffle')
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_toggle_shuffle')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'shuffle',
         ])
     player_shuffle.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+            ('my-host', 9000),
+            'office')
     assert 0 == result.exit_code
 
 
-def test_player_repeating_none(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_repeating_none(mocker, min_config, players_list_all):
     player_repeating = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_query_repeat',
-            return_value=squeezebox_cli.ui.main.Repeat.NONE)
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_query_repeat',
+            return_value=squeezebox_cli.player.Repeat.NONE)
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'repeating',
         ])
     player_repeating.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+            ('my-host', 9000),
+            'office')
     assert 0 == result.exit_code
     assert 'none\n' == result.output
 
 
-def test_player_repeating_song(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_repeating_song(mocker, min_config, players_list_all):
     player_repeating = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_query_repeat',
-            return_value=squeezebox_cli.ui.main.Repeat.SONG)
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_query_repeat',
+            return_value=squeezebox_cli.player.Repeat.SONG)
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'repeating',
         ])
     player_repeating.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+            ('my-host', 9000),
+            'office')
     assert 0 == result.exit_code
     assert 'song\n' == result.output
 
 
-def test_player_repeating_all(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_repeating_all(mocker, min_config, players_list_all):
     player_repeating = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_query_repeat',
-            return_value=squeezebox_cli.ui.main.Repeat.ALL)
+            'squeezebox_cli.ui.main'
+            '.squeezebox_cli.player.playlist_query_repeat',
+            return_value=squeezebox_cli.player.Repeat.ALL)
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'repeating',
         ])
     player_repeating.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+            ('my-host', 9000),
+            'office')
     assert 0 == result.exit_code
     assert 'all\n' == result.output
 
 
-def test_player_repeat_none(
-        mocker, min_config, telnet, player_from_index_or_name):
-    player_repeat = mocker.patch('squeezebox_cli.ui.main.playlist_set_repeat')
+def test_player_repeat_none(mocker, min_config, players_list_all):
+    player_repeat = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_set_repeat')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -454,15 +412,15 @@ def test_player_repeat_none(
         'none',
         ])
     player_repeat.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            squeezebox_cli.ui.main.Repeat.NONE)
+            ('my-host', 9000),
+            'office',
+            squeezebox_cli.player.Repeat.NONE)
     assert 0 == result.exit_code
 
 
-def test_player_repeat_song(
-        mocker, min_config, telnet, player_from_index_or_name):
-    player_repeat = mocker.patch('squeezebox_cli.ui.main.playlist_set_repeat')
+def test_player_repeat_song(mocker, min_config, players_list_all):
+    player_repeat = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_set_repeat')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -470,15 +428,15 @@ def test_player_repeat_song(
         'song',
         ])
     player_repeat.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            squeezebox_cli.ui.main.Repeat.SONG)
+            ('my-host', 9000),
+            'office',
+            squeezebox_cli.player.Repeat.SONG)
     assert 0 == result.exit_code
 
 
-def test_player_repeat_all(
-        mocker, min_config, telnet, player_from_index_or_name):
-    player_repeat = mocker.patch('squeezebox_cli.ui.main.playlist_set_repeat')
+def test_player_repeat_all(mocker, min_config, players_list_all):
+    player_repeat = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_set_repeat')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -486,31 +444,30 @@ def test_player_repeat_all(
         'all',
         ])
     player_repeat.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            squeezebox_cli.ui.main.Repeat.ALL)
+            ('my-host', 9000),
+            'office',
+            squeezebox_cli.player.Repeat.ALL)
     assert 0 == result.exit_code
 
 
-def test_player_toggle_repeat(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_toggle_repeat(mocker, min_config, players_list_all):
     player_repeat = mocker.patch(
-            'squeezebox_cli.ui.main.playlist_toggle_repeat')
+                'squeezebox_cli.ui.main'
+                '.squeezebox_cli.player.playlist_toggle_repeat')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'repeat',
         ])
     player_repeat.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+            ('my-host', 9000),
+            'office')
     assert 0 == result.exit_code
 
 
-def test_player_play_track(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_play_track(mocker, min_config, players_list_all):
     player_play = mocker.patch(
-            'squeezebox_cli.ui.main.play')
+            'squeezebox_cli.ui.main.squeezebox_cli.player.play')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -518,16 +475,15 @@ def test_player_play_track(
         'track',
         '1234',
         ])
-    player_play.assert_called_once_with(telnet.return_value,
-                                        player_from_index_or_name.return_value,
+    player_play.assert_called_once_with(('my-host', 9000),
+                                        'office',
                                         track_id=1234)
     assert 0 == result.exit_code
 
 
-def test_player_play_album(
-        mocker, min_config, telnet, player_from_index_or_name):
+def test_player_play_album(mocker, min_config, players_list_all):
     player_play = mocker.patch(
-            'squeezebox_cli.ui.main.play')
+            'squeezebox_cli.ui.main.squeezebox_cli.player.play')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -535,42 +491,38 @@ def test_player_play_album(
         'album',
         '1234',
         ])
-    player_play.assert_called_once_with(telnet.return_value,
-                                        player_from_index_or_name.return_value,
+    player_play.assert_called_once_with(('my-host', 9000),
+                                        'office',
                                         album_id=1234)
     assert 0 == result.exit_code
 
 
-def test_player_pause(mocker, min_config, telnet, player_from_index_or_name):
+def test_player_pause(mocker, min_config, players_list_all):
     player_pause = mocker.patch(
-            'squeezebox_cli.ui.main.pause')
+            'squeezebox_cli.ui.main.squeezebox_cli.player.pause')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'pause',
         ])
-    player_pause.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+    player_pause.assert_called_once_with(('my-host', 9000), 'office')
     assert 0 == result.exit_code
 
 
-def test_player_status(
-        min_config, telnet, player_from_index_or_name, player_status):
+def test_player_status(min_config, players_list_all, player_status):
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'status',
         ])
-    player_status.assert_called_once_with(
-            telnet.return_value, player_from_index_or_name.return_value)
+    player_status.assert_called_once_with(('my-host', 9000), 'office')
     assert ('[office] <stop> vol:53/100 2/5 the 1 [>>cardigan]\n' ==
             result.output)
     assert 0 == result.exit_code
 
 
 def test_player_status_long(
-        min_config, telnet, player_from_index_or_name, player_status,
+        min_config, players_list_all, player_status,
         database_songinfo):
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
@@ -578,9 +530,7 @@ def test_player_status_long(
         'status',
         '--long',
         ])
-    player_status.assert_called_once_with(
-            telnet.return_value, player_from_index_or_name.return_value)
-    print(result.output)
+    player_status.assert_called_once_with(('my-host', 9000), 'office')
     assert ('[office] <stop> vol:53/100 2/5 the 1 [>>cardigan]\n'
             '         index  title         album         artist\n'
             '-----  -------  ------------  ------------  ------------\n'
@@ -593,34 +543,33 @@ def test_player_status_long(
     assert 0 == result.exit_code
 
 
-def test_player_next(mocker, min_config, telnet, player_from_index_or_name):
-    player_next = mocker.patch('squeezebox_cli.ui.main.next')
+def test_player_next(mocker, min_config, players_list_all):
+    player_next = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.next')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'next',
         ])
-    player_next.assert_called_once_with(telnet.return_value,
-                                        player_from_index_or_name.return_value)
+    player_next.assert_called_once_with(('my-host', 9000), 'office')
     assert 0 == result.exit_code
 
 
 def test_player_previous(
-        mocker, min_config, telnet, player_from_index_or_name):
-    player_previous = mocker.patch('squeezebox_cli.ui.main.previous')
+        mocker, min_config, players_list_all):
+    player_previous = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.previous')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'previous',
         ])
-    player_previous.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value)
+    player_previous.assert_called_once_with(('my-host', 9000), 'office')
     assert 0 == result.exit_code
 
 
 def test_player_add_track(
-        min_config, telnet, player_from_index_or_name, playlist_add):
+        min_config, players_list_all, playlist_add):
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -629,14 +578,12 @@ def test_player_add_track(
         '1234',
         ])
     playlist_add.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            track_id='1234')
+            ('my-host', 9000), 'office', track_id='1234')
     assert 0 == result.exit_code
 
 
 def test_player_add_album(
-        min_config, telnet, player_from_index_or_name, playlist_add):
+        min_config, players_list_all, playlist_add):
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -645,15 +592,12 @@ def test_player_add_album(
         '1234',
         ])
     playlist_add.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            album_id='1234')
-    assert 0 == result.exit_code
+            ('my-host', 9000), 'office', album_id='1234')
     assert 0 == result.exit_code
 
 
 def test_player_insert_track(
-        min_config, telnet, player_from_index_or_name, playlist_insert):
+        min_config, players_list_all, playlist_insert):
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -662,14 +606,12 @@ def test_player_insert_track(
         '1234',
         ])
     playlist_insert.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            track_id='1234')
+            ('my-host', 9000), 'office', track_id='1234')
     assert 0 == result.exit_code
 
 
 def test_player_insert_album(
-        min_config, telnet, player_from_index_or_name, playlist_insert):
+        min_config, players_list_all, playlist_insert):
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -678,14 +620,12 @@ def test_player_insert_album(
         '1234',
         ])
     playlist_insert.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            album_id='1234')
+            ('my-host', 9000), 'office', album_id='1234')
     assert 0 == result.exit_code
 
 
 def test_player_volume_set(
-        min_config, telnet, player_from_index_or_name, player_set_volume):
+        min_config, players_list_all, player_set_volume):
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
@@ -693,24 +633,46 @@ def test_player_volume_set(
         '60',
         ])
     player_set_volume.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            '60')
+            ('my-host', 9000), 'office', '60')
     assert 0 == result.exit_code
 
 
 def test_playlist_remove(
-        mocker, min_config, telnet, player_from_index_or_name):
+        mocker, min_config, players_list_all):
     playlist_remove = mocker.patch(
-        'squeezebox_cli.ui.main.playlist_remove')
+        'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_remove')
     result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
         'player',
         'office',
         'remove',
         '60',
         ])
-    playlist_remove.assert_called_once_with(
-            telnet.return_value,
-            player_from_index_or_name.return_value,
-            59)
+    playlist_remove.assert_called_once_with(('my-host', 9000), 'office', 59)
+    assert 0 == result.exit_code
+
+
+def test_playlist_index(mocker, min_config, players_list_all):
+    playlist_index = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_index')
+    result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
+        'player',
+        'office',
+        'index',
+        '+3',
+        ])
+    playlist_index.assert_called_once_with(('my-host', 9000), 'office', '+3')
+    assert 0 == result.exit_code
+
+
+def test_playlist_index_negative(mocker, min_config, players_list_all):
+    playlist_index = mocker.patch(
+            'squeezebox_cli.ui.main.squeezebox_cli.player.playlist_index')
+    result = CliRunner().invoke(squeezebox_cli.ui.main.ui_main, [
+        'player',
+        'office',
+        'index',
+        '--',
+        '-3',
+        ])
+    playlist_index.assert_called_once_with(('my-host', 9000), 'office', '-3')
     assert 0 == result.exit_code
